YayMounts = LibStub("AceAddon-3.0"):NewAddon("YayMounts","AceConsole-3.0", "AceEvent-3.0")
AceGUI = LibStub("AceGUI-3.0")

local options = {
	name="YayMounts",
	handler = YayMounts,
	type = "group",
	args = {
		enable = {
			type = "execute",
			name = "Open GUI",
			func = "Frames",
		},
	}
}

local version = "1.0"
local t = { }
local collector = {}
local playerCombat
local _, class = UnitClass("player")
local faction = UnitFactionGroup("player")

local defaults = { 
	char = { 
		settings = {
			["DB"] = false, --database
			["ER"] = true, --emptyrandom: use random mount if none selected
			["HF"] = false, --have flying training
			["FFP"] = true, --Druid: Use flight form with priority
			["FBP"] = false, --Prioritize flying broom, even over Flight form
		},
		mounts = {
			['*'] = {
			},
		},
	}
}

function YayMounts:OnInitialize() 
	self.db = LibStub("AceDB-3.0"):New("YayMountsDB", defaults)
	local ymOptions = LibStub("AceConfigRegistry-3.0")
	ymOptions:RegisterOptionsTable("YayMounts",options)
	self.ymOpFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("YayMounts","YayMounts")
	self:RegisterChatCommand("ym", "ChatCommand")
end

function YayMounts:ChatCommand(input) --Chat command function
	if input == "reload" then
		YayMounts:SortMounts()
	else
		YayMounts:Frames()
	end
end


function YayMounts:OnEnable()-- addon enabled
	if not self.db.char.settings["DB"] then
		YayMounts:SortMounts()
	end
	YayMounts:MapOverrides()
 	
	self:RegisterEvent("UPDATE_BINDINGS", "MapOverrides")
	self:RegisterEvent("COMPANION_LEARNED", "SortMounts")
	if class ==  "DRUID" or class == "SHAMAN" then
		self:RegisterEvent("PLAYER_REGEN_DISABLED", "EventHandler")
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "EventHandler")
	end
end

--------------------------------------------------------------
--database handling and creation functions
--------------------------------------------------------------
function YayMounts:SpellToName(spell) --converts spell ID to Name
	local name = GetSpellInfo(spell)
	return name
end

function YayMounts:GetMounts(info, key)
	if info == "mounts" then
		return self.db.char.mounts[key][5]
	elseif info == "settings" then
		return self.db.char.settings[key]
	end
end

function YayMounts:SetMounts(info, key, state)
	if info == "mounts" then
		self.db.char.mounts[key][5] = state
	elseif info == "settings" then
		self.db.char.settings[key] = state
	end
end

function YayMounts:ClearMounts(mountType)-- clear all mounts
	for q=1,GetNumCompanions("MOUNT"), 1 do
		local _,_,S,_,_ = GetCompanionInfo("Mount", q)
		if self.db.char.mounts[S][4] == mountType then
			self.db.char.mounts[S][5] = false
			t[S]:SetValue(false)
		end
	end
end

function YayMounts:SelectAllMounts(mountType)-- select all mounts
	for q=1,GetNumCompanions("MOUNT"), 1 do
		local _,_,S,_,_ = GetCompanionInfo("Mount", q)
		if self.db.char.mounts[S][4] == mountType then
			self.db.char.mounts[S][5] = true
			t[S]:SetValue(true)
		end
	end
end

function YayMounts:BuildList(mountType)--Builds and draws the checkboxes for the opened tab
	for q=1, GetNumCompanions("MOUNT"), 1 do
	local _,N,S,_,_ = GetCompanionInfo("Mount", q)
		if self.db.char.mounts[S][4] == mountType then
			t[S] = AceGUI:Create("CheckBox")
			t[S]:SetLabel(N)
			t[S]:SetValue(YayMounts:GetMounts("mounts", S))
			t[S]:SetCallback("OnValueChanged",  function() YayMounts:SetMounts("mounts", S, t[S]:GetValue()) end)
			content:AddChild(t[S])
		end
	end
end

function YayMounts:SortMounts()--Rebuilds the database. Activates on load TODO CODE ON EVENT LEARN MOUNT
	for q=1, GetNumCompanions("MOUNT"), 1 do
		local X,_,S,I,O = GetCompanionInfo("MOUNT", q)
		local N = YayMounts:SpellToName(S)
		local mountType = nil
		for k,v in pairs(YayMountsType[faction]) do
			if N == v[1] then
				mountType = v[2]
				if mountType == "fly" then
					self.db.char.settings["HF"] = true
				end
			end
		end
		if mountType == nil then
			self:Print("New mount found: "..N)
		else
			if self.db.char.mounts[S][5] == nil then
				self.db.char.mounts[S] = {N, X, I, mountType, true}
			elseif self.db.char.mounts[S][5] == true then
				self.db.char.mounts[S] = {N, X, I, mountType, true}
			else
				self.db.char.mounts[S] = {N, X, I, mountType, false}
			end
		end
	end
	self.db.char.settings["DB"] = true
end

--------------------------------------------------------------
--GUI functions for drawing frames
--------------------------------------------------------------
local function DrawGroup1a(container) --Mounts, Tab1
	container:SetLayout("Flow")
	
	selectallbutton = AceGUI:Create("Button")
	selectallbutton:SetText("Select All")
	selectallbutton:SetCallback("OnClick", function() YayMounts:SelectAllMounts("ground") end)
	container:AddChild(selectallbutton)
		
	clearallbutton = AceGUI:Create("Button")
	clearallbutton:SetText("Clear All")
	clearallbutton:SetCallback("OnClick", function() YayMounts:ClearMounts("ground") end)
	container:AddChild(clearallbutton)
	
	scrollcontainerframe = AceGUI:Create("SimpleGroup")
	scrollcontainerframe:SetLayout("Fill")
	scrollcontainerframe:SetFullWidth(true)
	scrollcontainerframe:SetFullHeight(true)
	container:AddChild(scrollcontainerframe)

	scrollframe = AceGUI:Create("ScrollFrame")
	scrollcontainerframe:AddChild(scrollframe)
	
	content = AceGUI:Create("InlineGroup")
	content:SetFullWidth(true)
 	content:SetLayout("Flow")

	scrollframe:AddChild(content)
	YayMounts:BuildList("ground")
end

local function DrawGroup2a(container) --Mounts, Tab2
	container:SetLayout("Flow")
	
	selectallbutton = AceGUI:Create("Button")
	selectallbutton:SetText("Select All")
	selectallbutton:SetCallback("OnClick", function() YayMounts:SelectAllMounts("fly") end)
	container:AddChild(selectallbutton)
		
	clearallbutton = AceGUI:Create("Button")
	clearallbutton:SetText("Clear All")
	clearallbutton:SetCallback("OnClick", function() YayMounts:ClearMounts("fly") end)
	container:AddChild(clearallbutton)
	
	scrollcontainerframe = AceGUI:Create("SimpleGroup")
	scrollcontainerframe:SetLayout("Fill")
	scrollcontainerframe:SetFullWidth(true)
	scrollcontainerframe:SetFullHeight(true)
	container:AddChild(scrollcontainerframe)

	scrollframe = AceGUI:Create("ScrollFrame")
	scrollcontainerframe:AddChild(scrollframe)
	
	content = AceGUI:Create("InlineGroup")
	content:SetFullWidth(true)
 	content:SetLayout("Flow")

	scrollframe:AddChild(content)
	YayMounts:BuildList("fly")
end

local function DrawGroup3a(container) --Mounts, Tab3
	container:SetLayout("Flow")
	
	selectallbutton = AceGUI:Create("Button")
	selectallbutton:SetText("Select All")
	selectallbutton:SetCallback("OnClick", function() YayMounts:SelectAllMounts("scaling") end)
	container:AddChild(selectallbutton)
		
	clearallbutton = AceGUI:Create("Button")
	clearallbutton:SetText("Clear All")
	clearallbutton:SetCallback("OnClick", function() YayMounts:ClearMounts("scaling") end)
	container:AddChild(clearallbutton)
	
	scrollcontainerframe = AceGUI:Create("SimpleGroup")
	scrollcontainerframe:SetLayout("Fill")
	scrollcontainerframe:SetFullWidth(true)
	scrollcontainerframe:SetFullHeight(true)
	container:AddChild(scrollcontainerframe)

	scrollframe = AceGUI:Create("ScrollFrame")
	scrollcontainerframe:AddChild(scrollframe)
	
	content = AceGUI:Create("InlineGroup")
	content:SetFullWidth(true)
 	content:SetLayout("Flow")

	scrollframe:AddChild(content)
	YayMounts:BuildList("scaling")
end

local function SelectGroupMounts(container, event, group)
	container:ReleaseChildren()
	if group == "tab1" then
		DrawGroup1a(container)
	elseif group == "tab2" then
		DrawGroup2a(container)
	elseif group == "tab3" then
		DrawGroup3a(container)
	end
end

local function DrawGroup1(container)-- function that draws the widgets for the first tab MOUNTS
	container:SetLayout("Fill")

	local tab  = AceGUI:Create("TabGroup")
	tab:SetLayout("Flow")
	tab:SetTabs({{text="Ground", value="tab1"}, {text="Flying", value="tab2"}, {text="Scaling", value="tab3"}})
	tab:SetCallback("OnGroupSelected", SelectGroupMounts)
	tab:SelectTab("tab1")
	container:AddChild(tab)
end


local function DrawGroup3(container)-- function that draws the widgets for the third tab OPTIONS
	container:SetLayout("List")
	
	emptyRandom = AceGUI:Create("CheckBox")
	emptyRandom:SetLabel("Choose random usable mount if no usable mounts enabled")
	emptyRandom:SetValue(YayMounts:GetMounts("settings", "ER"))
	emptyRandom:SetCallback("OnValueChanged",  function() YayMounts:SetMounts("settings", "ER", emptyRandom:GetValue()) end)
	emptyRandom:SetFullWidth(true)
	container:AddChild(emptyRandom)
	
	flightFormPriority = AceGUI:Create("CheckBox")
	flightFormPriority:SetLabel("Prioritize using Flight Form over a regular mount when not moving")
	flightFormPriority:SetValue(YayMounts:GetMounts("settings", "FFP"))
	flightFormPriority:SetCallback("OnValueChanged",  function() YayMounts:SetMounts("settings", "FFP", flightFormPriority:GetValue()) end)
	flightFormPriority:SetFullWidth(true)
	container:AddChild(flightFormPriority)
	
	flyingBroomPriority = AceGUI:Create("CheckBox")
	if class == "DRUID" then
		flyingBroomPriority:SetLabel("Prioritize using Flying Broom over a regular mount when not moving, even over Flight Form")
	else
		flyingBroomPriority:SetLabel("Prioritize using Flying Broom over a regular mount when not moving")
	end
	flyingBroomPriority:SetValue(YayMounts:GetMounts("settings", "FBP"))
	flyingBroomPriority:SetCallback("OnValueChanged", function() YayMounts:SetMounts("settings", "FBP", flyingBroomPriority:GetValue()) end)
	flyingBroomPriority:SetFullWidth(true)
	container:AddChild(flyingBroomPriority)
end

local function SelectGroup(container, event, group)-- Callback function for OnGroupSelected
	container:ReleaseChildren()
	if group == "tab1" then
		DrawGroup1(container)
--~ 	elseif group == "tab2" then
--~ 		DrawGroup2(container)
	elseif group == "tab3" then
		DrawGroup3(container)
	end
end

function YayMounts:Frames()-- Basic frame
	if not yayFrame then
		yayFrame = AceGUI:Create("Frame")
		yayFrame:SetCallback("OnClose", function(widget) AceGUI:Release(widget); yayFrame = nil end)
		yayFrame:SetTitle("Yay Mounts")
		yayFrame:SetStatusText("Version "..version..", by Cyrae - Windrunner US")
		yayFrame:SetLayout("Fill")
		yayFrame:SetWidth(720)
		yayFrame:SetHeight(490)
	
		local tab  = AceGUI:Create("TabGroup")
		tab:SetLayout("Flow")
		tab:SetTabs({{text="Mounts", value="tab1"}, {text="Options", value="tab3"}})
		tab:SetCallback("OnGroupSelected", SelectGroup)
		tab:SelectTab("tab1")
		yayFrame:AddChild(tab)
	end
end

--------------------------------------------------------------
--Mounting and button click options
--------------------------------------------------------------
function YayMounts:Mounting(t, zone)
	local e = {}
	local i = 1
	local mountType
	
	if t ~= nil then
		mountType = t
	elseif IsFlyableArea() and self.db.char.settings["HF"] then
		mountType = "fly"
	elseif IsOutdoors() then
		if  zone == "Wintergrasp" and GetWintergraspWaitTime() ~= nil then
			mountType = "fly"
		else
			mountType = "ground"
		end
	else
		mountType = "ground"
	end
	
	if mountType == "fly" and self.db.char.settings["FFP"] and class == "DRUID" then
		if IsUsableSpell("Flight Form") then
			return "Flight Form"
		elseif IsUsableSpell("Swift Flight Form") then
			return "Swift Flight Form"	
		end
	end
	
	--write code to prefer scaling mounts over ground mounts when underwater, unless in vashjir?
	for k,v in pairs(self.db.char.mounts) do
		if (v[4] == mountType or v[4] == "scaling") and v[5] then
			e[i] = v[1]
			i = i + 1
		end
	end
	
	e = YayMounts:Filter(e, zone)
	
	if #e == 0 and self.db.char.settings["ER"] then
		i = 1
		for k,v in pairs(self.db.char.mounts) do
			if (v[4] == mountType or v[4] == "scaling") then
				e[i] = v[1]
				i = i + 1
			end
		end
		e = YayMounts:Filter(e, zone)
	elseif #e == 0 then
		self:Print("No mounts Selected")
		return nil
	end
	return e[math.random(#e)]
end

function YayMounts:PreClick()
local zone = GetRealZoneText()
local flyable = IsFlyableArea()
	if zone == "Wintergrasp" and GetWintergraspWaitTime() ~= nil then
		flyable = true
	end

	if IsMounted() then
		Dismount()
	elseif CanExitVehicle() then
		VehicleExit()
	elseif GetItemCount("Flying Broom", false) > 0 and flyable then
		if not playerCombat and (GetUnitSpeed("player") ~= 0 or IsFalling() or IsFlying()) then
			YayMounts:SetMacroButton("Flying Broom")
		elseif not playerCombat and self.db.char.settings["FBP"] == true then
			YayMounts:SetMacroButton("Flying Broom")
		else
			YayMounts:SetMacroButton(YayMounts:Mounting("fly", zone))
		end
	elseif (GetUnitSpeed("player") ~= 0 or playerCombat) and (class == "DRUID") then
		if IsSwimming() and IsUsableSpell("Aquatic Form") then
			YayMounts:SetMacroButton("Aquatic Form")
		elseif flyable and (IsFalling() or IsFlying()) and not playerCombat then
			if IsUsableSpell("Flight Form") then
				YayMounts:SetMacroButton("Flight Form")
			elseif IsUsableSpell("Swift Flight Form") then
				YayMounts:SetMacroButton("Swift Flight Form")
			end
		elseif IsUsableSpell("Travel Form") and not IsFalling() then
			YayMounts:SetMacroButton("Travel Form")
		end
	elseif (GetUnitSpeed("player") ~= 0 or playerCombat or not IsOutdoors()) and (class == "SHAMAN") and IsUsableSpell("Ghost Wolf") then	
		YayMounts:SetMacroButton("Ghost Wolf")
	elseif IsSwimming() then
		local timer, _,_,scale,_,_ = GetMirrorTimerInfo(2)
		local name, _, _, _, _, _, _, _, _, _, _ = UnitBuff("player", "Water Breathing")
		if 	not name and (timer == "UNKNOWN" or scale >=0) then
			YayMounts:SetMacroButton(YayMounts:Mounting("fly", zone))
		else
			YayMounts:SetMacroButton(YayMounts:Mounting("ground", zone))
		end
	elseif zone == "The Oculus" and YayMounts:Filter(_, zone) then
		return
	else
		YayMounts:SetMacroButton(YayMounts:Mounting(nil, zone))		
	end
end

function YayMounts:SetMacroButton(mount)
	if mount then
		secureButton:SetAttribute("macrotext", "/use "..mount)
	else
		secureButton:SetAttribute("macrotext", nil)
	end
end

function YayMounts:MapOverrides()
	if not InCombatLockdown() then
		key1, key2 = GetBindingKey("YAYMOUNTBINDING")
		if key1 then
			SetOverrideBindingClick(secureButton, true, key1, secureButton:GetName())
		end
		if key2 then
			SetOverrideBindingClick(secureButton, true, key2, secureButton:GetName())
		end
	end
end

function YayMounts:Filter(e, zone)
	if zone ~= "Temple of Ahn'Qiraj" and e ~= _ then
		for k,v in pairs(e) do
			if e[k] == YayMountsType["Ahn'Qiraj"][v] then
				tremove(e, k)
			end
		end
		return e
	elseif zone == "The Oculus" then
		if GetItemCount("Amber Essence") == 1 then
			YayMounts:SetMacroButton("Amber Essence")
			return true
		elseif GetItemCount("Ruby Essence") == 1 then
			YayMounts:SetMacroButton("Ruby Essence")
			return true
		elseif GetItemCount("Emerald Essence") == 1 then
			YayMounts:SetMacroButton("Emerald Essence")
			return true
		end
	end
end

function YayMounts:EventHandler(arg1)
	if  arg1 == "PLAYER_REGEN_DISABLED" then
		playerCombat = true
		if class == "SHAMAN" then
			YayMounts:SetMacroButton("Ghost Wolf")
		elseif class == "DRUID" then
			YayMounts:SetMacroButton("[swimming] Aquatic Form; Travel Form")
		end
	elseif arg1 == "PLAYER_REGEN_ENABLED" then
		playerCombat = false
	end
end